@@ -61,7 +61,7 @@
 //!
 //! ## Supported Languages
 //!
-//! ### Permissively Licensed (97 languages, included by default)
+//! ### Permissively Licensed (101 languages, included by default)
 //!
 //! | Language | Feature Flag | License |
 //! |----------|--------------|---------|
@@ -88,10 +88,16 @@
 
 //! | Cap&#039;n Proto | `lang-capnp` | MIT |
 
+//! | Cedar | `lang-cedar` | MIT OR Apache-2.0 |
+
+//! | Cedar Schema | `lang-cedarschema` | MIT OR Apache-2.0 |
+
 //! | Clojure | `lang-clojure` | Unlicense |
 
 //! | CMake | `lang-cmake` | MIT |
 
+//! | COBOL | `lang-cobol` | MIT |
+
 //! | Common Lisp | `lang-commonlisp` | MIT |
 
 //! | C++ | `lang-cpp` | MIT |
@@ -156,8 +162,6 @@
 
 //! | Julia | `lang-julia` | MIT |
 
-//! | KDL | `lang-kdl` | MIT OR Apache-2.0 |
-
 //! | Kotlin | `lang-kotlin` | MIT |
 
 //! | Lean | `lang-lean` | MIT |
@@ -208,6 +212,8 @@
 
 //! | SCSS | `lang-scss` | MIT |
 
+//! | Solidity | `lang-solidity` | MIT |
+
 //! | SPARQL | `lang-sparql` | MIT |
 
 //! | SQL | `lang-sql` | MIT |
@@ -216,6 +222,8 @@
 
 //! | Starlark | `lang-starlark` | MIT |
 
+//! | Styx | `lang-styx` | MIT OR Apache-2.0 |
+
 //! | Svelte | `lang-svelte` | MIT |
 
 //! | Swift | `lang-swift` | MIT |
@@ -411,6 +419,12 @@ pub fn detect_language(path: &str) -> Option<&'static str> {
 
         "capnp" => "capnp",
 
+        "cbl" => "cobol",
+
+        "cedar" => "cedar",
+
+        "cedarschema" => "cedarschema",
+
         "cfg" => "ini",
 
         "cjs" => "javascript",
@@ -425,12 +439,18 @@ pub fn detect_language(path: &str) -> Option<&'static str> {
 
         "cmd" => "batch",
 
+        "cob" => "cobol",
+
+        "cobol" => "cobol",
+
         "commonlisp" => "commonlisp",
 
         "conf" => "ini",
 
         "cpp" => "cpp",
 
+        "cpy" => "cobol",
+
         "cs" => "c-sharp",
 
         "csharp" => "c-sharp",
@@ -551,8 +571,6 @@ pub fn detect_language(path: &str) -> Option<&'static str> {
 
         "julia" => "julia",
 
-        "kdl" => "kdl",
-
         "kotlin" => "kotlin",
 
         "kt" => "kotlin",
@@ -677,6 +695,10 @@ pub fn detect_language(path: &str) -> Option<&'static str> {
 
         "shell" => "bash",
 
+        "sol" => "solidity",
+
+        "solidity" => "solidity",
+
         "sparql" => "sparql",
 
         "sql" => "sql",
@@ -689,6 +711,8 @@ pub fn detect_language(path: &str) -> Option<&'static str> {
 
         "starlark" => "starlark",
 
+        "styx" => "styx",
+
         "sv" => "verilog",
 
         "svelte" => "svelte",
@@ -835,6 +859,14 @@ pub use arborium_caddy as lang_caddy;
 pub use arborium_capnp as lang_capnp;
 
 
+#[cfg(feature = "lang-cedar")]
+pub use arborium_cedar as lang_cedar;
+
+
+#[cfg(feature = "lang-cedarschema")]
+pub use arborium_cedarschema as lang_cedarschema;
+
+
 #[cfg(feature = "lang-clojure")]
 pub use arborium_clojure as lang_clojure;
 
@@ -843,6 +875,10 @@ pub use arborium_clojure as lang_clojure;
 pub use arborium_cmake as lang_cmake;
 
 
+#[cfg(feature = "lang-cobol")]
+pub use arborium_cobol as lang_cobol;
+
+
 #[cfg(feature = "lang-commonlisp")]
 pub use arborium_commonlisp as lang_commonlisp;
 
@@ -971,10 +1007,6 @@ pub use arborium_json as lang_json;
 pub use arborium_julia as lang_julia;
 
 
-#[cfg(feature = "lang-kdl")]
-pub use arborium_kdl as lang_kdl;
-
-
 #[cfg(feature = "lang-kotlin")]
 pub use arborium_kotlin as lang_kotlin;
 
@@ -1079,6 +1111,10 @@ pub use arborium_scheme as lang_scheme;
 pub use arborium_scss as lang_scss;
 
 
+#[cfg(feature = "lang-solidity")]
+pub use arborium_solidity as lang_solidity;
+
+
 #[cfg(feature = "lang-sparql")]
 pub use arborium_sparql as lang_sparql;
 
@@ -1095,6 +1131,10 @@ pub use arborium_ssh_config as lang_ssh_config;
 pub use arborium_starlark as lang_starlark;
 
 
+#[cfg(feature = "lang-styx")]
+pub use arborium_styx as lang_styx;
+
+
 #[cfg(feature = "lang-svelte")]
 pub use arborium_svelte as lang_svelte;
 
@@ -1182,3 +1222,335 @@ pub use arborium_zig as lang_zig;
 #[cfg(feature = "lang-zsh")]
 pub use arborium_zsh as lang_zsh;
 
+
+
+/// Returns the tree-sitter `Language` for the given language name.
+///
+/// This function only returns languages that are enabled via feature flags.
+/// If no grammar matches the provided language name, it returns `None`.
+///
+/// # Example
+///
+/// ```rust,ignore
+/// use arborium::get_language;
+///
+/// // Returns Some if the "lang-rust" feature is enabled
+/// if let Some(lang) = get_language("rust") {
+///     println!("Got Rust language: {:?}", lang);
+/// }
+///
+/// // Returns None for unknown languages
+/// assert!(get_language("unknown-language").is_none());
+/// ```
+pub fn get_language(name: &str) -> Option<tree_sitter::Language> {
+    match name {
+
+        #[cfg(feature = "lang-ada")]
+        "ada" => Some(arborium_ada::language().into()),
+
+        #[cfg(feature = "lang-agda")]
+        "agda" => Some(arborium_agda::language().into()),
+
+        #[cfg(feature = "lang-asciidoc")]
+        "asciidoc" => Some(arborium_asciidoc::language().into()),
+
+        #[cfg(feature = "lang-asm")]
+        "asm" => Some(arborium_asm::language().into()),
+
+        #[cfg(feature = "lang-awk")]
+        "awk" => Some(arborium_awk::language().into()),
+
+        #[cfg(feature = "lang-bash")]
+        "bash" => Some(arborium_bash::language().into()),
+
+        #[cfg(feature = "lang-batch")]
+        "batch" => Some(arborium_batch::language().into()),
+
+        #[cfg(feature = "lang-c")]
+        "c" => Some(arborium_c::language().into()),
+
+        #[cfg(feature = "lang-c-sharp")]
+        "c-sharp" => Some(arborium_c_sharp::language().into()),
+
+        #[cfg(feature = "lang-caddy")]
+        "caddy" => Some(arborium_caddy::language().into()),
+
+        #[cfg(feature = "lang-capnp")]
+        "capnp" => Some(arborium_capnp::language().into()),
+
+        #[cfg(feature = "lang-cedar")]
+        "cedar" => Some(arborium_cedar::language().into()),
+
+        #[cfg(feature = "lang-cedarschema")]
+        "cedarschema" => Some(arborium_cedarschema::language().into()),
+
+        #[cfg(feature = "lang-clojure")]
+        "clojure" => Some(arborium_clojure::language().into()),
+
+        #[cfg(feature = "lang-cmake")]
+        "cmake" => Some(arborium_cmake::language().into()),
+
+        #[cfg(feature = "lang-cobol")]
+        "cobol" => Some(arborium_cobol::language().into()),
+
+        #[cfg(feature = "lang-commonlisp")]
+        "commonlisp" => Some(arborium_commonlisp::language().into()),
+
+        #[cfg(feature = "lang-cpp")]
+        "cpp" => Some(arborium_cpp::language().into()),
+
+        #[cfg(feature = "lang-css")]
+        "css" => Some(arborium_css::language().into()),
+
+        #[cfg(feature = "lang-d")]
+        "d" => Some(arborium_d::language().into()),
+
+        #[cfg(feature = "lang-dart")]
+        "dart" => Some(arborium_dart::language().into()),
+
+        #[cfg(feature = "lang-devicetree")]
+        "devicetree" => Some(arborium_devicetree::language().into()),
+
+        #[cfg(feature = "lang-diff")]
+        "diff" => Some(arborium_diff::language().into()),
+
+        #[cfg(feature = "lang-dockerfile")]
+        "dockerfile" => Some(arborium_dockerfile::language().into()),
+
+        #[cfg(feature = "lang-dot")]
+        "dot" => Some(arborium_dot::language().into()),
+
+        #[cfg(feature = "lang-elisp")]
+        "elisp" => Some(arborium_elisp::language().into()),
+
+        #[cfg(feature = "lang-elixir")]
+        "elixir" => Some(arborium_elixir::language().into()),
+
+        #[cfg(feature = "lang-elm")]
+        "elm" => Some(arborium_elm::language().into()),
+
+        #[cfg(feature = "lang-erlang")]
+        "erlang" => Some(arborium_erlang::language().into()),
+
+        #[cfg(feature = "lang-fish")]
+        "fish" => Some(arborium_fish::language().into()),
+
+        #[cfg(feature = "lang-fsharp")]
+        "fsharp" => Some(arborium_fsharp::language().into()),
+
+        #[cfg(feature = "lang-gleam")]
+        "gleam" => Some(arborium_gleam::language().into()),
+
+        #[cfg(feature = "lang-glsl")]
+        "glsl" => Some(arborium_glsl::language().into()),
+
+        #[cfg(feature = "lang-go")]
+        "go" => Some(arborium_go::language().into()),
+
+        #[cfg(feature = "lang-graphql")]
+        "graphql" => Some(arborium_graphql::language().into()),
+
+        #[cfg(feature = "lang-groovy")]
+        "groovy" => Some(arborium_groovy::language().into()),
+
+        #[cfg(feature = "lang-haskell")]
+        "haskell" => Some(arborium_haskell::language().into()),
+
+        #[cfg(feature = "lang-hcl")]
+        "hcl" => Some(arborium_hcl::language().into()),
+
+        #[cfg(feature = "lang-hlsl")]
+        "hlsl" => Some(arborium_hlsl::language().into()),
+
+        #[cfg(feature = "lang-html")]
+        "html" => Some(arborium_html::language().into()),
+
+        #[cfg(feature = "lang-idris")]
+        "idris" => Some(arborium_idris::language().into()),
+
+        #[cfg(feature = "lang-ini")]
+        "ini" => Some(arborium_ini::language().into()),
+
+        #[cfg(feature = "lang-java")]
+        "java" => Some(arborium_java::language().into()),
+
+        #[cfg(feature = "lang-javascript")]
+        "javascript" => Some(arborium_javascript::language().into()),
+
+        #[cfg(feature = "lang-jinja2")]
+        "jinja2" => Some(arborium_jinja2::language().into()),
+
+        #[cfg(feature = "lang-jq")]
+        "jq" => Some(arborium_jq::language().into()),
+
+        #[cfg(feature = "lang-json")]
+        "json" => Some(arborium_json::language().into()),
+
+        #[cfg(feature = "lang-julia")]
+        "julia" => Some(arborium_julia::language().into()),
+
+        #[cfg(feature = "lang-kotlin")]
+        "kotlin" => Some(arborium_kotlin::language().into()),
+
+        #[cfg(feature = "lang-lean")]
+        "lean" => Some(arborium_lean::language().into()),
+
+        #[cfg(feature = "lang-lua")]
+        "lua" => Some(arborium_lua::language().into()),
+
+        #[cfg(feature = "lang-markdown")]
+        "markdown" => Some(arborium_markdown::language().into()),
+
+        #[cfg(feature = "lang-matlab")]
+        "matlab" => Some(arborium_matlab::language().into()),
+
+        #[cfg(feature = "lang-meson")]
+        "meson" => Some(arborium_meson::language().into()),
+
+        #[cfg(feature = "lang-nginx")]
+        "nginx" => Some(arborium_nginx::language().into()),
+
+        #[cfg(feature = "lang-ninja")]
+        "ninja" => Some(arborium_ninja::language().into()),
+
+        #[cfg(feature = "lang-nix")]
+        "nix" => Some(arborium_nix::language().into()),
+
+        #[cfg(feature = "lang-objc")]
+        "objc" => Some(arborium_objc::language().into()),
+
+        #[cfg(feature = "lang-ocaml")]
+        "ocaml" => Some(arborium_ocaml::language().into()),
+
+        #[cfg(feature = "lang-perl")]
+        "perl" => Some(arborium_perl::language().into()),
+
+        #[cfg(feature = "lang-php")]
+        "php" => Some(arborium_php::language().into()),
+
+        #[cfg(feature = "lang-postscript")]
+        "postscript" => Some(arborium_postscript::language().into()),
+
+        #[cfg(feature = "lang-powershell")]
+        "powershell" => Some(arborium_powershell::language().into()),
+
+        #[cfg(feature = "lang-prolog")]
+        "prolog" => Some(arborium_prolog::language().into()),
+
+        #[cfg(feature = "lang-python")]
+        "python" => Some(arborium_python::language().into()),
+
+        #[cfg(feature = "lang-query")]
+        "query" => Some(arborium_query::language().into()),
+
+        #[cfg(feature = "lang-r")]
+        "r" => Some(arborium_r::language().into()),
+
+        #[cfg(feature = "lang-rescript")]
+        "rescript" => Some(arborium_rescript::language().into()),
+
+        #[cfg(feature = "lang-ron")]
+        "ron" => Some(arborium_ron::language().into()),
+
+        #[cfg(feature = "lang-ruby")]
+        "ruby" => Some(arborium_ruby::language().into()),
+
+        #[cfg(feature = "lang-rust")]
+        "rust" => Some(arborium_rust::language().into()),
+
+        #[cfg(feature = "lang-scala")]
+        "scala" => Some(arborium_scala::language().into()),
+
+        #[cfg(feature = "lang-scheme")]
+        "scheme" => Some(arborium_scheme::language().into()),
+
+        #[cfg(feature = "lang-scss")]
+        "scss" => Some(arborium_scss::language().into()),
+
+        #[cfg(feature = "lang-solidity")]
+        "solidity" => Some(arborium_solidity::language().into()),
+
+        #[cfg(feature = "lang-sparql")]
+        "sparql" => Some(arborium_sparql::language().into()),
+
+        #[cfg(feature = "lang-sql")]
+        "sql" => Some(arborium_sql::language().into()),
+
+        #[cfg(feature = "lang-ssh-config")]
+        "ssh-config" => Some(arborium_ssh_config::language().into()),
+
+        #[cfg(feature = "lang-starlark")]
+        "starlark" => Some(arborium_starlark::language().into()),
+
+        #[cfg(feature = "lang-styx")]
+        "styx" => Some(arborium_styx::language().into()),
+
+        #[cfg(feature = "lang-svelte")]
+        "svelte" => Some(arborium_svelte::language().into()),
+
+        #[cfg(feature = "lang-swift")]
+        "swift" => Some(arborium_swift::language().into()),
+
+        #[cfg(feature = "lang-textproto")]
+        "textproto" => Some(arborium_textproto::language().into()),
+
+        #[cfg(feature = "lang-thrift")]
+        "thrift" => Some(arborium_thrift::language().into()),
+
+        #[cfg(feature = "lang-tlaplus")]
+        "tlaplus" => Some(arborium_tlaplus::language().into()),
+
+        #[cfg(feature = "lang-toml")]
+        "toml" => Some(arborium_toml::language().into()),
+
+        #[cfg(feature = "lang-tsx")]
+        "tsx" => Some(arborium_tsx::language().into()),
+
+        #[cfg(feature = "lang-typescript")]
+        "typescript" => Some(arborium_typescript::language().into()),
+
+        #[cfg(feature = "lang-typst")]
+        "typst" => Some(arborium_typst::language().into()),
+
+        #[cfg(feature = "lang-uiua")]
+        "uiua" => Some(arborium_uiua::language().into()),
+
+        #[cfg(feature = "lang-vb")]
+        "vb" => Some(arborium_vb::language().into()),
+
+        #[cfg(feature = "lang-verilog")]
+        "verilog" => Some(arborium_verilog::language().into()),
+
+        #[cfg(feature = "lang-vhdl")]
+        "vhdl" => Some(arborium_vhdl::language().into()),
+
+        #[cfg(feature = "lang-vim")]
+        "vim" => Some(arborium_vim::language().into()),
+
+        #[cfg(feature = "lang-vue")]
+        "vue" => Some(arborium_vue::language().into()),
+
+        #[cfg(feature = "lang-wit")]
+        "wit" => Some(arborium_wit::language().into()),
+
+        #[cfg(feature = "lang-x86asm")]
+        "x86asm" => Some(arborium_x86asm::language().into()),
+
+        #[cfg(feature = "lang-xml")]
+        "xml" => Some(arborium_xml::language().into()),
+
+        #[cfg(feature = "lang-yaml")]
+        "yaml" => Some(arborium_yaml::language().into()),
+
+        #[cfg(feature = "lang-yuri")]
+        "yuri" => Some(arborium_yuri::language().into()),
+
+        #[cfg(feature = "lang-zig")]
+        "zig" => Some(arborium_zig::language().into()),
+
+        #[cfg(feature = "lang-zsh")]
+        "zsh" => Some(arborium_zsh::language().into()),
+
+        _ => None,
+    }
+}
\ No newline at end of file
