@@ -20,7 +20,7 @@ use crate::{
         Builder as SubjectTokenBuilder, SubjectToken, SubjectTokenProvider,
     },
 };
-use gax::error::CredentialsError;
+use google_cloud_gax::error::CredentialsError;
 use serde::{Deserialize, Serialize};
 use std::{
     fmt::{Display, Formatter},
@@ -178,7 +178,7 @@ impl ExecutableSourcedCredentials {
         Self::parse_token(output)
     }
 
-    /// Parses a full command string into a command and its arguments.    
+    /// Parses a full command string into a command and its arguments.
     fn split_command(command: String) -> (String, Vec<String>) {
         let mut parts = command.split_whitespace();
         let Some(cmd) = parts.next() else {
@@ -241,7 +241,7 @@ mod tests {
     use crate::constants::JWT_TOKEN_TYPE;
     use scoped_env::ScopedEnv;
     use serde_json::json;
-    use serial_test::serial;
+    use serial_test::{parallel, serial};
     use std::error::Error;
     use std::time::{Duration, SystemTime, UNIX_EPOCH};
     use test_case::test_case;
@@ -323,7 +323,7 @@ mod tests {
     }
 
     #[tokio::test]
-    #[serial]
+    #[parallel]
     async fn read_valid_token_from_output_file() -> TestResult {
         let expiration = SystemTime::now().duration_since(UNIX_EPOCH)?;
         let expiration = expiration + Duration::from_secs(3600);
@@ -495,7 +495,7 @@ done";
             .expect_err("should fail with timeout");
 
         assert!(err.is_transient());
-        assert!(err.source().is_some());
+        assert!(err.source().is_some(), "{err:?}");
 
         let source_err = err.source().unwrap();
         assert!(source_err.to_string().contains("deadline"));
