@@ -12,7 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-//! Obtain [OIDC ID Tokens].
+//! Obtain, use, and verify [OIDC ID Tokens].
 //!
 //! `IDTokenCredentials` obtain OIDC ID tokens, which are commonly
 //! used for [service to service authentication]. For example, when the
@@ -53,24 +53,26 @@
 //! ## Example: Verifying an ID token
 //!
 //! Within the receiving private service, you can parse the authorization header to
-//! receive the information being sent by the Bearer token and use the [Verifier] to
-//! check if is valid.
+//! receive the information being sent by the Bearer token and use the
+//! [Verifier][verifier::Verifier] to check if the token is valid.
 //!
-//! ```no_run
+//! ```
 //! # use google_cloud_auth::credentials::idtoken;
+//! # use google_cloud_auth::credentials::idtoken::verifier::Verifier;
 //! # use std::time::Duration;
+//! # tokio_test::block_on(async {
 //! let audience = "https://my-service.a.run.app";
 //! let verifier = idtoken::verifier::Builder::new([audience]).build();
 //!
-//! async fn verify_id_token(token: &str) -> anyhow::Result<()> {
+//! async fn verify_id_token(verifier: &Verifier, token: &str) -> anyhow::Result<()> {
 //!     let claims = verifier.verify(token).await?;
-//!     let email = claims["email"].as_str()?;
 //!
-//!     println!("Hello: {:?}", email);
+//!     println!("Hello: {:?}", claims["email"]);
 //! #   Ok(())
 //! }
+//! # });
 //! ```
-//!
+//! [Verifier]: https://docs.rs/google-cloud-auth/latest/google_cloud_auth/credentials/idtoken/struct.Verifier.html
 //! [OIDC ID Tokens]: https://cloud.google.com/docs/authentication/token-types#identity-tokens
 //! [Service to Service Authentication]: https://cloud.google.com/run/docs/authenticating/service-to-service
 
@@ -79,7 +81,7 @@ use crate::credentials::{AdcContents, CredentialsError, extract_credential_type,
 use crate::token::Token;
 use crate::{BuildResult, Result};
 use base64::{Engine as _, engine::general_purpose::URL_SAFE_NO_PAD};
-pub use serde_json::{Map, Value};
+use serde_json::Value;
 use std::collections::HashMap;
 use std::future::Future;
 use std::sync::Arc;
@@ -332,9 +334,12 @@ fn instant_from_epoch_seconds(secs: u64, now: SystemTime) -> Option<Instant> {
 #[cfg(test)]
 pub(crate) mod tests {
     use super::*;
-    use jsonwebtoken::{Algorithm, EncodingKey, Header};
+    use crate::credentials::service_account::jws::JwsHeader;
     use mds::Format;
-    use rsa::pkcs1::EncodeRsaPrivateKey;
+    use p256::ecdsa::signature::Signer;
+    use p256::ecdsa::{Signature, SigningKey};
+    use rsa::Pkcs1v15Sign;
+    use rsa::sha2::{Digest, Sha256};
     use serde_json::json;
     use serial_test::parallel;
     use std::collections::HashMap;
@@ -365,8 +370,11 @@ pub(crate) mod tests {
         let now = now.duration_since(UNIX_EPOCH).unwrap();
         let then = now + DEFAULT_TEST_TOKEN_EXPIRATION;
 
-        let mut header = Header::new(Algorithm::RS256);
-        header.kid = Some(TEST_KEY_ID.to_string());
+        let header = JwsHeader {
+            alg: "RS256",
+            typ: "JWT",
+            kid: Some(TEST_KEY_ID.to_string()),
+        };
 
         let mut claims: HashMap<&str, Value> = HashMap::new();
         claims.insert("aud", Value::String(audience));
@@ -378,13 +386,46 @@ pub(crate) mod tests {
             claims.insert(k, v);
         }
 
-        let private_cert = crate::credentials::tests::RSA_PRIVATE_KEY
-            .to_pkcs1_der()
-            .expect("Failed to encode private key to PKCS#1 DER");
+        let key = crate::credentials::tests::RSA_PRIVATE_KEY.clone();
 
-        let private_key = EncodingKey::from_rsa_der(private_cert.as_bytes());
+        let encoded_header = header.encode().unwrap();
+        let encoded_claims = URL_SAFE_NO_PAD.encode(serde_json::to_string(&claims).unwrap());
 
-        jsonwebtoken::encode(&header, &claims, &private_key).expect("failed to encode jwt")
+        let to_sign = format!("{}.{}", encoded_header, encoded_claims);
+        let digest = Sha256::digest(to_sign.as_bytes());
+        let sig = key
+            .sign(Pkcs1v15Sign::new::<Sha256>(), &digest)
+            .expect("Failed to sign");
+        let encoded_sig = URL_SAFE_NO_PAD.encode(sig);
+
+        format!("{}.{}", to_sign, encoded_sig)
+    }
+
+    pub(crate) fn generate_test_id_token_es256(audience: &str) -> String {
+        let now = SystemTime::now().duration_since(UNIX_EPOCH).unwrap();
+        let then = now + DEFAULT_TEST_TOKEN_EXPIRATION;
+        let header = JwsHeader {
+            alg: "ES256",
+            typ: "JWT",
+            kid: Some(TEST_KEY_ID.to_string()),
+        };
+        let mut claims: HashMap<&str, Value> = HashMap::new();
+        claims.insert("aud", Value::String(audience.to_string()));
+        claims.insert("iss", "accounts.google.com".into());
+        claims.insert("exp", then.as_secs().into());
+        claims.insert("iat", now.as_secs().into());
+
+        let private_key = crate::credentials::tests::ES256_PRIVATE_KEY.clone();
+        let key = SigningKey::from(private_key);
+
+        let encoded_header = header.encode().unwrap();
+        let encoded_claims = URL_SAFE_NO_PAD.encode(serde_json::to_string(&claims).unwrap());
+
+        let to_sign = format!("{}.{}", encoded_header, encoded_claims);
+        let sig: Signature = key.sign(to_sign.as_bytes());
+        let encoded_sig = URL_SAFE_NO_PAD.encode(sig.to_bytes());
+
+        format!("{}.{}", to_sign, encoded_sig)
     }
 
     #[tokio::test(start_paused = true)]
@@ -397,7 +438,7 @@ pub(crate) mod tests {
         let token = parse_id_token_from_str_impl(id_token.clone(), now)?;
 
         assert_eq!(token.token, id_token);
-        assert!(token.expires_at.is_some());
+        assert!(token.expires_at.is_some(), "{token:?}");
 
         let expires_at = token.expires_at.unwrap();
         let expiration = expires_at.duration_since(Instant::now());
@@ -427,7 +468,7 @@ pub(crate) mod tests {
         });
 
         let result = build_id_token_credentials(audience, false, Some(json));
-        assert!(result.is_err());
+        assert!(result.is_err(), "{result:?}");
         let err = result.unwrap_err();
         assert!(err.is_not_supported());
         assert!(
@@ -455,7 +496,7 @@ pub(crate) mod tests {
         });
 
         let result = build_id_token_credentials(audience, false, Some(json));
-        assert!(result.is_err());
+        assert!(result.is_err(), "{result:?}");
         let err = result.unwrap_err();
         assert!(err.is_not_supported());
         assert!(err.to_string().contains("external_account"));
@@ -471,7 +512,7 @@ pub(crate) mod tests {
         });
 
         let result = build_id_token_credentials(audience, false, Some(json));
-        assert!(result.is_err());
+        assert!(result.is_err(), "{result:?}");
         let err = result.unwrap_err();
         assert!(err.is_unknown_type());
         assert!(err.to_string().contains("unknown_credential_type"));
