@@ -20,17 +20,17 @@
 //!
 //! ## Example: Verifying an ID token
 //!
-//! ```no_run
+//! ```
 //! # use google_cloud_auth::credentials::idtoken;
+//! # use google_cloud_auth::credentials::idtoken::verifier::Verifier;
 //! # use std::time::Duration;
 //! let audience = "https://my-service.a.run.app";
 //! let verifier = idtoken::verifier::Builder::new([audience]).build();
 //!
-//! async fn verify_my_token(token: &str) -> anyhow::Result<()> {
+//! async fn verify_my_token(verifier: &Verifier, token: &str) -> anyhow::Result<()> {
 //!     let claims = verifier.verify(token).await?;
-//!     let email = claims["email"].as_str()?;
 //!
-//!     println!("Hello: {:?}", email);
+//!     println!("Hello: {:?}", claims["email"]);
 //! #   Ok(())
 //! }
 //! ```
@@ -38,7 +38,10 @@
 
 use crate::credentials::internal::jwk_client::JwkClient;
 use jsonwebtoken::Validation;
-pub use serde_json::{Map, Value};
+/// Represents the claims in an ID token.
+pub use serde_json::Map;
+/// Represents a claim value in an ID token.
+pub use serde_json::Value;
 use std::time::Duration;
 
 /// Builder is used construct a [Verifier] of id tokens.
@@ -98,7 +101,7 @@ impl Builder {
     /// # use google_cloud_auth::credentials::idtoken::verifier::Builder;
     /// let verifier = Builder::new(["https://my-service.a.run.app"])
     ///     .with_jwks_url("https://www.googleapis.com/oauth2/v3/certs")
-    ///     .build();    
+    ///     .build();
     /// ```
     pub fn with_jwks_url<S: Into<String>>(mut self, jwks_url: S) -> Self {
         self.jwks_url = Some(jwks_url.into());
@@ -113,7 +116,7 @@ impl Builder {
     /// # Example
     ///
     /// ```
-    /// # use google_cloud_auth::credentials::idtoken::Builder;
+    /// # use google_cloud_auth::credentials::idtoken::verifier::Builder;
     /// # use std::time::Duration;
     /// let verifier = Builder::new(["https://my-service.a.run.app"])
     ///     .with_clock_skew(Duration::from_secs(60))
@@ -124,7 +127,7 @@ impl Builder {
         self
     }
 
-    /// Verifies the ID token and returns the claims.
+    /// Returns a [Verifier] instance with the configured settings.
     pub fn build(self) -> Verifier {
         Verifier {
             jwk_client: JwkClient::new(),
@@ -143,7 +146,6 @@ impl Builder {
 /// ```
 /// # use google_cloud_auth::credentials::idtoken::verifier::Builder;
 /// # use std::time::Duration;
-///
 /// async fn verify_id_token(token: &str) {
 ///     let verifier = Builder::new(["https://my-service.a.run.app"]).build();
 ///
@@ -236,12 +238,7 @@ impl Error {
 
     /// A problem validating JWT token accordingly to set criteria.
     pub fn is_invalid(&self) -> bool {
-        matches!(self.0, ErrorKind::Invalid(_))
-    }
-
-    /// A problem validating JWT token on a specific field.
-    pub fn is_invalid_field(&self) -> bool {
-        matches!(self.0, ErrorKind::InvalidField(_, _))
+        matches!(self.0, ErrorKind::Invalid(_)) || matches!(self.0, ErrorKind::InvalidField(_, _))
     }
 
     /// A problem fetching certificates to validate the JWT token.
@@ -250,7 +247,7 @@ impl Error {
     }
 
     /// A problem to decode the JWT token.
-    pub(crate) fn decode<T>(source: T) -> Error
+    fn decode<T>(source: T) -> Error
     where
         T: Into<BoxError>,
     {
@@ -258,7 +255,7 @@ impl Error {
     }
 
     /// A problem fetching certificates to validate the JWT token.
-    pub(crate) fn load_cert<T>(source: T) -> Error
+    fn load_cert<T>(source: T) -> Error
     where
         T: Into<BoxError>,
     {
@@ -266,7 +263,7 @@ impl Error {
     }
 
     /// Validation error of the JWT Token.
-    pub(crate) fn invalid<T>(source: T) -> Error
+    fn invalid<T>(source: T) -> Error
     where
         T: Into<BoxError>,
     {
@@ -274,7 +271,7 @@ impl Error {
     }
 
     /// Validation error of the JWT Token on a specific field.
-    pub(crate) fn invalid_field<S: Into<String>, T>(field: S, source: T) -> Error
+    fn invalid_field<S: Into<String>, T>(field: S, source: T) -> Error
     where
         T: Into<BoxError>,
     {
@@ -298,40 +295,25 @@ enum ErrorKind {
 pub(crate) mod tests {
     use super::*;
     use crate::credentials::idtoken::tests::{
-        TEST_KEY_ID, generate_test_id_token, generate_test_id_token_with_claims,
+        generate_test_id_token, generate_test_id_token_es256, generate_test_id_token_with_claims,
+    };
+    use crate::credentials::internal::jwk_client::tests::{
+        create_es256_jwk_set_response, create_rsa256_jwk_set_response,
     };
-    use base64::Engine;
     use httptest::matchers::{all_of, request};
     use httptest::responders::{json_encoded, status_code};
     use httptest::{Expectation, Server};
     use jsonwebtoken::{Algorithm, EncodingKey, Header};
     use rsa::pkcs1::EncodeRsaPrivateKey;
-    use rsa::traits::PublicKeyParts;
     use std::collections::HashMap;
     use std::time::{Duration, SystemTime, UNIX_EPOCH};
 
     type TestResult = anyhow::Result<()>;
 
-    fn create_jwk_set_response() -> serde_json::Value {
-        let pub_cert = crate::credentials::tests::RSA_PRIVATE_KEY.to_public_key();
-        serde_json::json!({
-            "keys": [
-                {
-                    "e": base64::engine::general_purpose::URL_SAFE_NO_PAD.encode(pub_cert.e().to_bytes_be()),
-                    "kid": TEST_KEY_ID,
-                    "use": "sig",
-                    "kty": "RSA",
-                    "n": base64::engine::general_purpose::URL_SAFE_NO_PAD.encode(pub_cert.n().to_bytes_be()),
-                    "alg": "RS256"
-                }
-            ]
-        })
-    }
-
     #[tokio::test]
     async fn test_verify_success() -> TestResult {
         let server = Server::run();
-        let response = create_jwk_set_response();
+        let response = create_rsa256_jwk_set_response();
         server.expect(
             Expectation::matching(all_of![request::path("/certs"),])
                 .times(1)
@@ -347,10 +329,10 @@ pub(crate) mod tests {
             .build();
 
         let claims = verifier.verify(token).await?;
-        assert!(!claims.is_empty());
+        assert!(!claims.is_empty(), "{token:?}");
 
         let claims = verifier.verify(token).await?;
-        assert!(!claims.is_empty());
+        assert!(!claims.is_empty(), "{token:?}");
 
         Ok(())
     }
@@ -361,7 +343,7 @@ pub(crate) mod tests {
         server.expect(
             Expectation::matching(all_of![request::path("/certs"),])
                 .times(1)
-                .respond_with(json_encoded(create_jwk_set_response())),
+                .respond_with(json_encoded(create_rsa256_jwk_set_response())),
         );
 
         let audience = "https://example.com";
@@ -373,8 +355,8 @@ pub(crate) mod tests {
             .build();
 
         let result = verifier.verify(token).await;
-        assert!(result.is_err());
-        assert!(result.unwrap_err().is_invalid_field());
+        assert!(result.is_err(), "{result:?}");
+        assert!(result.unwrap_err().is_invalid());
 
         Ok(())
     }
@@ -385,7 +367,7 @@ pub(crate) mod tests {
         server.expect(
             Expectation::matching(all_of![request::path("/certs"),])
                 .times(1)
-                .respond_with(json_encoded(create_jwk_set_response())),
+                .respond_with(json_encoded(create_rsa256_jwk_set_response())),
         );
 
         let audiences = ["https://example.com", "https://another_example.com"];
@@ -397,7 +379,7 @@ pub(crate) mod tests {
             let token = generate_test_id_token(audience);
             let token = token.as_str();
             let claims = verifier.verify(token).await?;
-            assert!(!claims.is_empty());
+            assert!(!claims.is_empty(), "{token:?}");
         }
 
         Ok(())
@@ -409,7 +391,7 @@ pub(crate) mod tests {
         server.expect(
             Expectation::matching(all_of![request::path("/certs"),])
                 .times(1)
-                .respond_with(json_encoded(create_jwk_set_response())),
+                .respond_with(json_encoded(create_rsa256_jwk_set_response())),
         );
 
         let audience = "https://example.com";
@@ -423,8 +405,8 @@ pub(crate) mod tests {
             .build();
 
         let result = verifier.verify(token).await;
-        assert!(result.is_err());
-        assert!(result.unwrap_err().is_invalid_field());
+        assert!(result.is_err(), "{result:?}");
+        assert!(result.unwrap_err().is_invalid());
 
         Ok(())
     }
@@ -435,7 +417,7 @@ pub(crate) mod tests {
         server.expect(
             Expectation::matching(all_of![request::path("/certs"),])
                 .times(1)
-                .respond_with(json_encoded(create_jwk_set_response())),
+                .respond_with(json_encoded(create_rsa256_jwk_set_response())),
         );
 
         let audience = "https://example.com";
@@ -451,9 +433,7 @@ pub(crate) mod tests {
             .with_email(email)
             .build();
 
-        let result = verifier.verify(token).await;
-        assert!(result.is_ok());
-        let claims = result.unwrap();
+        let claims = verifier.verify(token).await?;
         assert_eq!(claims["email"].as_str().unwrap(), email);
 
         Ok(())
@@ -465,7 +445,7 @@ pub(crate) mod tests {
         server.expect(
             Expectation::matching(all_of![request::path("/certs"),])
                 .times(1)
-                .respond_with(json_encoded(create_jwk_set_response())),
+                .respond_with(json_encoded(create_rsa256_jwk_set_response())),
         );
 
         let audience = "https://example.com";
@@ -482,8 +462,8 @@ pub(crate) mod tests {
             .build();
 
         let result = verifier.verify(token).await;
-        assert!(result.is_err());
-        assert!(result.unwrap_err().is_invalid_field());
+        assert!(result.is_err(), "{result:?}");
+        assert!(result.unwrap_err().is_invalid());
         Ok(())
     }
 
@@ -493,7 +473,7 @@ pub(crate) mod tests {
         server.expect(
             Expectation::matching(all_of![request::path("/certs"),])
                 .times(1)
-                .respond_with(json_encoded(create_jwk_set_response())),
+                .respond_with(json_encoded(create_rsa256_jwk_set_response())),
         );
 
         let audience = "https://example.com";
@@ -508,7 +488,7 @@ pub(crate) mod tests {
             .build();
 
         let result = verifier.verify(token).await;
-        assert!(result.is_err());
+        assert!(result.is_err(), "{result:?}");
         assert!(result.unwrap_err().is_invalid());
 
         Ok(())
@@ -520,7 +500,7 @@ pub(crate) mod tests {
         server.expect(
             Expectation::matching(all_of![request::path("/certs"),])
                 .times(1)
-                .respond_with(json_encoded(create_jwk_set_response())),
+                .respond_with(json_encoded(create_rsa256_jwk_set_response())),
         );
 
         let audience = "https://example.com";
@@ -537,8 +517,8 @@ pub(crate) mod tests {
             .build();
 
         let result = verifier.verify(token).await;
-        assert!(result.is_err());
-        assert!(result.unwrap_err().is_invalid_field());
+        assert!(result.is_err(), "{result:?}");
+        assert!(result.unwrap_err().is_invalid());
         Ok(())
     }
 
@@ -548,7 +528,7 @@ pub(crate) mod tests {
         server.expect(
             Expectation::matching(all_of![request::path("/certs"),])
                 .times(1)
-                .respond_with(json_encoded(create_jwk_set_response())),
+                .respond_with(json_encoded(create_rsa256_jwk_set_response())),
         );
 
         let audience = "https://example.com";
@@ -560,11 +540,10 @@ pub(crate) mod tests {
 
         let verifier = Builder::new([audience])
             .with_jwks_url(format!("http://{}/certs", server.addr()))
-            .with_clock_skew(Duration::from_secs(10))
+            .with_clock_skew(Duration::from_secs(60))
             .build();
 
-        let result = verifier.verify(token).await;
-        assert!(result.is_ok());
+        let _claims = verifier.verify(token).await?;
 
         Ok(())
     }
@@ -576,7 +555,7 @@ pub(crate) mod tests {
         let invalid_token = "invalid.token.format";
 
         let result = verifier.verify(invalid_token).await;
-        assert!(result.is_err());
+        assert!(result.is_err(), "{result:?}");
         assert!(result.unwrap_err().is_decode());
 
         Ok(())
@@ -599,8 +578,8 @@ pub(crate) mod tests {
         let verifier = Builder::new(["https://example.com"]).build();
 
         let result = verifier.verify(&token).await;
-        assert!(result.is_err());
-        assert!(result.unwrap_err().is_invalid_field());
+        assert!(result.is_err(), "{result:?}");
+        assert!(result.unwrap_err().is_invalid());
 
         Ok(())
     }
@@ -623,9 +602,32 @@ pub(crate) mod tests {
             .build();
 
         let result = verifier.verify(token).await;
-        assert!(result.is_err());
+        assert!(result.is_err(), "{result:?}");
         assert!(result.unwrap_err().is_load_cert());
 
         Ok(())
     }
+
+    #[tokio::test]
+    async fn test_verify_es256_success() -> TestResult {
+        let server = Server::run();
+        let response = create_es256_jwk_set_response();
+        server.expect(
+            Expectation::matching(all_of![request::path("/certs"),])
+                .times(1)
+                .respond_with(json_encoded(response.clone())),
+        );
+
+        let audience = "https://example.com";
+        let token = generate_test_id_token_es256(audience);
+
+        let verifier = Builder::new([audience])
+            .with_jwks_url(format!("http://{}/certs", server.addr()))
+            .build();
+
+        let claims = verifier.verify(&token).await?;
+        assert!(!claims.is_empty(), "{token:?}");
+
+        Ok(())
+    }
 }
