@@ -61,8 +61,8 @@
 //! # use std::time::Duration;
 //! # use http::Extensions;
 //! # tokio_test::block_on(async {
-//! use gax::retry_policy::{AlwaysRetry, RetryPolicyExt};
-//! use gax::exponential_backoff::ExponentialBackoff;
+//! use google_cloud_gax::retry_policy::{AlwaysRetry, RetryPolicyExt};
+//! use google_cloud_gax::exponential_backoff::ExponentialBackoff;
 //! # let source_credentials = json!({
 //! #     "type": "authorized_user",
 //! #     "client_id": "test-client-id",
@@ -93,22 +93,23 @@
 
 use crate::build_errors::Error as BuilderError;
 use crate::constants::DEFAULT_SCOPE;
-use crate::credentials::dynamic::CredentialsProvider;
+use crate::credentials::dynamic::{AccessTokenCredentialsProvider, CredentialsProvider};
 use crate::credentials::{
-    CacheableResource, Credentials, build_credentials, extract_credential_type,
+    AccessToken, AccessTokenCredentials, CacheableResource, Credentials, build_credentials,
+    extract_credential_type,
 };
 use crate::errors::{self, CredentialsError};
 use crate::headers_util::{
-    self, ACCESS_TOKEN_REQUEST_TYPE, build_cacheable_headers, metrics_header_value,
+    self, ACCESS_TOKEN_REQUEST_TYPE, AuthHeadersBuilder, metrics_header_value,
 };
 use crate::retry::{Builder as RetryTokenProviderBuilder, TokenProviderWithRetry};
 use crate::token::{CachedTokenProvider, Token, TokenProvider};
 use crate::token_cache::TokenCache;
 use crate::{BuildResult, Result};
 use async_trait::async_trait;
-use gax::backoff_policy::BackoffPolicyArg;
-use gax::retry_policy::RetryPolicyArg;
-use gax::retry_throttler::RetryThrottlerArg;
+use google_cloud_gax::backoff_policy::BackoffPolicyArg;
+use google_cloud_gax::retry_policy::RetryPolicyArg;
+use google_cloud_gax::retry_throttler::RetryThrottlerArg;
 use http::{Extensions, HeaderMap};
 use reqwest::Client;
 use serde_json::Value;
@@ -122,6 +123,7 @@ pub(crate) const IMPERSONATED_CREDENTIAL_TYPE: &str = "imp";
 pub(crate) const DEFAULT_LIFETIME: Duration = Duration::from_secs(3600);
 pub(crate) const MSG: &str = "failed to fetch token";
 
+#[derive(Debug, Clone)]
 pub(crate) enum BuilderSource {
     FromJson(Value),
     FromCredentials(Credentials),
@@ -343,7 +345,7 @@ impl Builder {
     /// # use google_cloud_auth::credentials::impersonated::Builder;
     /// # use serde_json::json;
     /// # tokio_test::block_on(async {
-    /// use gax::retry_policy::{AlwaysRetry, RetryPolicyExt};
+    /// use google_cloud_gax::retry_policy::{AlwaysRetry, RetryPolicyExt};
     /// let impersonated_credential = json!({ /* add details here */ });
     /// let credentials = Builder::new(impersonated_credential.into())
     ///     .with_retry_policy(AlwaysRetry.with_attempt_limit(3))
@@ -364,7 +366,7 @@ impl Builder {
     /// # use serde_json::json;
     /// # use std::time::Duration;
     /// # tokio_test::block_on(async {
-    /// use gax::exponential_backoff::ExponentialBackoff;
+    /// use google_cloud_gax::exponential_backoff::ExponentialBackoff;
     /// let policy = ExponentialBackoff::default();
     /// let impersonated_credential = json!({ /* add details here */ });
     /// let credentials = Builder::new(impersonated_credential.into())
@@ -392,7 +394,7 @@ impl Builder {
     /// # use google_cloud_auth::credentials::impersonated::Builder;
     /// # use serde_json::json;
     /// # tokio_test::block_on(async {
-    /// use gax::retry_throttler::AdaptiveThrottler;
+    /// use google_cloud_gax::retry_throttler::AdaptiveThrottler;
     /// let impersonated_credential = json!({ /* add details here */ });
     /// let credentials = Builder::new(impersonated_credential.into())
     ///     .with_retry_throttler(AdaptiveThrottler::default())
@@ -421,8 +423,53 @@ impl Builder {
     ///
     /// [application-default credentials]: https://cloud.google.com/docs/authentication/application-default-credentials
     pub fn build(self) -> BuildResult<Credentials> {
+        Ok(self.build_access_token_credentials()?.into())
+    }
+
+    /// Returns an [AccessTokenCredentials] instance with the configured settings.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # use google_cloud_auth::credentials::impersonated::Builder;
+    /// # use google_cloud_auth::credentials::{AccessTokenCredentials, AccessTokenCredentialsProvider};
+    /// # use serde_json::json;
+    /// # tokio_test::block_on(async {
+    /// let impersonated_credential = json!({
+    ///     "type": "impersonated_service_account",
+    ///     "service_account_impersonation_url": "https://iamcredentials.googleapis.com/v1/projects/-/serviceAccounts/test-principal:generateAccessToken",
+    ///     "source_credentials": {
+    ///         "type": "authorized_user",
+    ///         "client_id": "test-client-id",
+    ///         "client_secret": "test-client-secret",
+    ///         "refresh_token": "test-refresh-token"
+    ///     }
+    /// });
+    /// let credentials: AccessTokenCredentials = Builder::new(impersonated_credential.into())
+    ///     .build_access_token_credentials()?;
+    /// let access_token = credentials.access_token().await?;
+    /// println!("Token: {}", access_token.token);
+    /// # Ok::<(), anyhow::Error>(())
+    /// # });
+    /// ```
+    ///
+    /// # Errors
+    ///
+    /// Returns a [BuilderError] for one of the following cases:
+    /// - If the `impersonated_service_account` provided to [`Builder::new`] cannot
+    ///   be successfully deserialized into the expected format. This typically happens
+    ///   if the JSON value is malformed or missing required fields. For more information,
+    ///   on how to generate `impersonated_service_account` json, consult the relevant
+    ///   section in the [application-default credentials] guide.
+    /// - If the `impersonated_service_account` provided to [`Builder::new`] has a
+    ///   `source_credentials` of `impersonated_service_account` type.
+    /// - If `service_account_impersonation_url` is not provided after initializing
+    ///   the builder with [`Builder::from_source_credentials`].
+    ///
+    /// [application-default credentials]: https://cloud.google.com/docs/authentication/application-default-credentials
+    pub fn build_access_token_credentials(self) -> BuildResult<AccessTokenCredentials> {
         let (token_provider, quota_project_id) = self.build_components()?;
-        Ok(Credentials {
+        Ok(AccessTokenCredentials {
             inner: Arc::new(ImpersonatedServiceAccount {
                 token_provider: TokenCache::new(token_provider),
                 quota_project_id,
@@ -430,6 +477,80 @@ impl Builder {
         })
     }
 
+    /// Returns a [crate::signer::Signer] instance with the configured settings.
+    ///
+    /// The returned [crate::signer::Signer] uses the [IAM signBlob API] to sign content. This API
+    /// requires a network request for each signing operation.
+    ///
+    /// Note that if the source credentials are a service account, the signer
+    /// might perform signing locally. For more details on local signing, see
+    /// [crate::credentials::service_account::Builder::build_signer].
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # use google_cloud_auth::credentials::impersonated::Builder;
+    /// # use google_cloud_auth::signer::Signer;
+    /// # use serde_json::json;
+    /// # tokio_test::block_on(async {
+    /// let impersonated_credential = json!({
+    ///     "type": "impersonated_service_account",
+    ///     "service_account_impersonation_url": "https://iamcredentials.googleapis.com/v1/projects/-/serviceAccounts/test-principal:generateAccessToken",
+    ///     "source_credentials": {
+    ///         "type": "authorized_user",
+    ///         "client_id": "test-client-id",
+    ///         "client_secret": "test-client-secret",
+    ///         "refresh_token": "test-refresh-token"
+    ///     }
+    /// });
+    ///
+    /// let signer: Signer = Builder::new(impersonated_credential).build_signer()?;
+    /// # Ok::<(), anyhow::Error>(())
+    /// # });
+    /// ```
+    ///
+    /// [IAM signBlob API]: https://cloud.google.com/iam/docs/reference/credentials/rest/v1/projects.serviceAccounts/signBlob
+    pub fn build_signer(self) -> BuildResult<crate::signer::Signer> {
+        self.build_signer_with_iam_endpoint_override(None)
+    }
+
+    // only used for testing
+    fn build_signer_with_iam_endpoint_override(
+        self,
+        iam_endpoint: Option<String>,
+    ) -> BuildResult<crate::signer::Signer> {
+        let source = self.source.clone();
+        match source {
+            BuilderSource::FromJson(json) => {
+                let signer = build_signer_from_json(json.clone())?;
+                if let Some(signer) = signer {
+                    return Ok(signer);
+                }
+                let components = build_components_from_json(json)?;
+                let client_email =
+                    extract_client_email(&components.service_account_impersonation_url)?;
+                let creds = self.build()?;
+                let signer = crate::signer::iam::IamSigner::new(client_email, creds, iam_endpoint);
+                Ok(crate::signer::Signer {
+                    inner: Arc::new(signer),
+                })
+            }
+            BuilderSource::FromCredentials(source_credentials) => {
+                let components = build_components_from_credentials(
+                    source_credentials,
+                    self.service_account_impersonation_url.clone(),
+                )?;
+                let client_email =
+                    extract_client_email(&components.service_account_impersonation_url)?;
+                let creds = self.build()?;
+                let signer = crate::signer::iam::IamSigner::new(client_email, creds, iam_endpoint);
+                Ok(crate::signer::Signer {
+                    inner: Arc::new(signer),
+                })
+            }
+        }
+    }
+
     fn build_components(
         self,
     ) -> BuildResult<(
@@ -493,7 +614,7 @@ pub(crate) fn build_components_from_json(
     // the quota project and they typically need different scopes.
     // If user does want some specific scopes or quota, they can build using the
     // from_source_credentials method.
-    let source_credentials = build_credentials(Some(config.source_credentials), None, None)?;
+    let source_credentials = build_credentials(Some(config.source_credentials), None, None)?.into();
 
     Ok(ImpersonatedCredentialComponents {
         source_credentials,
@@ -504,6 +625,44 @@ pub(crate) fn build_components_from_json(
     })
 }
 
+// Check if the source credentials is a service account and if so, build a signer from it.
+// Service account signers can do signing locally, which is more efficient than using the
+// remote/API based signer.
+fn build_signer_from_json(json: Value) -> BuildResult<Option<crate::signer::Signer>> {
+    use crate::credentials::service_account::ServiceAccountKey;
+    use crate::signer::service_account::ServiceAccountSigner;
+
+    let config =
+        serde_json::from_value::<ImpersonatedConfig>(json).map_err(BuilderError::parsing)?;
+
+    let client_email = extract_client_email(&config.service_account_impersonation_url)?;
+    let source_credential_type = extract_credential_type(&config.source_credentials)?;
+    if source_credential_type == "service_account" {
+        let service_account_key =
+            serde_json::from_value::<ServiceAccountKey>(config.source_credentials)
+                .map_err(BuilderError::parsing)?;
+        let signing_provider = ServiceAccountSigner::from_impersonated_service_account(
+            service_account_key,
+            client_email,
+        );
+        let signer = crate::signer::Signer {
+            inner: Arc::new(signing_provider),
+        };
+        return Ok(Some(signer));
+    }
+    Ok(None)
+}
+
+fn extract_client_email(service_account_impersonation_url: &str) -> BuildResult<String> {
+    let mut parts = service_account_impersonation_url.split("/serviceAccounts/");
+    match (parts.nth(1), parts.next()) {
+        (Some(email), None) => Ok(email.trim_end_matches(":generateAccessToken").to_string()),
+        _ => Err(BuilderError::parsing(
+            "invalid service account impersonation URL",
+        )),
+    }
+}
+
 pub(crate) fn build_components_from_credentials(
     source_credentials: Credentials,
     service_account_impersonation_url: Option<String>,
@@ -547,7 +706,21 @@ where
 {
     async fn headers(&self, extensions: Extensions) -> Result<CacheableResource<HeaderMap>> {
         let token = self.token_provider.token(extensions).await?;
-        build_cacheable_headers(&token, &self.quota_project_id)
+
+        AuthHeadersBuilder::new(&token)
+            .maybe_quota_project_id(self.quota_project_id.as_deref())
+            .build()
+    }
+}
+
+#[async_trait::async_trait]
+impl<T> AccessTokenCredentialsProvider for ImpersonatedServiceAccount<T>
+where
+    T: CachedTokenProvider,
+{
+    async fn access_token(&self) -> Result<AccessToken> {
+        let token = self.token_provider.token(Extensions::new()).await?;
+        token.into()
     }
 }
 
@@ -1070,9 +1243,6 @@ mod tests {
         };
         let fmt = format!("{expected:?}");
         assert!(fmt.contains("UserCredentials"), "{fmt}");
-        assert!(fmt.contains("test-client-id"), "{fmt}");
-        assert!(!fmt.contains("test-client-secret"), "{fmt}");
-        assert!(!fmt.contains("test-refresh-token"), "{fmt}");
         assert!(fmt.contains("https://iamcredentials.googleapis.com/v1/projects/-/serviceAccounts/test-principal:generateAccessToken"), "{fmt}");
         assert!(fmt.contains("delegate1"), "{fmt}");
         assert!(fmt.contains("scope1"), "{fmt}");
@@ -1177,7 +1347,7 @@ mod tests {
         .unwrap();
 
         let result = Builder::from_source_credentials(source_credentials).build();
-        assert!(result.is_err());
+        assert!(result.is_err(), "{result:?}");
         let err = result.unwrap_err();
         assert!(err.is_parsing());
         assert!(
@@ -1204,7 +1374,7 @@ mod tests {
         });
 
         let result = Builder::new(nested_impersonated).build();
-        assert!(result.is_err());
+        assert!(result.is_err(), "{result:?}");
         let err = result.unwrap_err();
         assert!(err.is_parsing());
         assert!(
@@ -1222,7 +1392,7 @@ mod tests {
         });
 
         let result = Builder::new(malformed_impersonated).build();
-        assert!(result.is_err());
+        assert!(result.is_err(), "{result:?}");
         let err = result.unwrap_err();
         assert!(err.is_parsing());
         assert!(
@@ -1242,7 +1412,7 @@ mod tests {
         });
 
         let result = Builder::new(invalid_source).build();
-        assert!(result.is_err());
+        assert!(result.is_err(), "{result:?}");
         let err = result.unwrap_err();
         assert!(err.is_unknown_type());
     }
@@ -1460,6 +1630,51 @@ mod tests {
         Ok(())
     }
 
+    #[tokio::test]
+    async fn access_token_credentials_success() -> TestResult {
+        let server = Server::run();
+        server.expect(
+            Expectation::matching(request::method_path("POST", "/token")).respond_with(
+                json_encoded(json!({
+                    "access_token": "test-user-account-token",
+                    "expires_in": 3600,
+                    "token_type": "Bearer",
+                })),
+            ),
+        );
+        let expire_time = (OffsetDateTime::now_utc() + time::Duration::hours(1))
+            .format(&time::format_description::well_known::Rfc3339)
+            .unwrap();
+        server.expect(
+            Expectation::matching(request::method_path(
+                "POST",
+                "/v1/projects/-/serviceAccounts/test-principal:generateAccessToken",
+            ))
+            .respond_with(json_encoded(json!({
+                "accessToken": "test-impersonated-token",
+                "expireTime": expire_time
+            }))),
+        );
+
+        let impersonated_credential = json!({
+            "type": "impersonated_service_account",
+            "service_account_impersonation_url": server.url("/v1/projects/-/serviceAccounts/test-principal:generateAccessToken").to_string(),
+            "source_credentials": {
+                "type": "authorized_user",
+                "client_id": "test-client-id",
+                "client_secret": "test-client-secret",
+                "refresh_token": "test-refresh-token",
+                "token_uri": server.url("/token").to_string()
+            }
+        });
+        let creds = Builder::new(impersonated_credential).build_access_token_credentials()?;
+
+        let access_token = creds.access_token().await?;
+        assert_eq!(access_token.token, "test-impersonated-token");
+
+        Ok(())
+    }
+
     #[tokio::test]
     async fn test_with_target_principal() {
         let source_credentials = crate::credentials::user_account::Builder::new(json!({
@@ -1878,4 +2093,153 @@ mod tests {
         server.verify_and_clear();
         Ok(())
     }
+
+    #[tokio::test]
+    async fn test_impersonated_remote_signer() -> TestResult {
+        use base64::{Engine, prelude::BASE64_STANDARD};
+
+        let server = Server::run();
+        server.expect(
+            Expectation::matching(request::method_path("POST", "/token"))
+                .times(2..)
+                .respond_with(json_encoded(json!({
+                    "access_token": "test-user-account-token",
+                    "expires_in": 3600,
+                    "token_type": "Bearer",
+                }))),
+        );
+        let expire_time = (OffsetDateTime::now_utc() + time::Duration::hours(1))
+            .format(&time::format_description::well_known::Rfc3339)
+            .unwrap();
+        server.expect(
+            Expectation::matching(request::method_path(
+                "POST",
+                "/v1/projects/-/serviceAccounts/test-principal:generateAccessToken",
+            ))
+            .times(2)
+            .respond_with(json_encoded(json!({
+                "accessToken": "test-impersonated-token",
+                "expireTime": expire_time
+            }))),
+        );
+
+        server.expect(
+            Expectation::matching(all_of![
+                request::method_path(
+                    "POST",
+                    "/v1/projects/-/serviceAccounts/test-principal:signBlob"
+                ),
+                request::headers(contains((
+                    "authorization",
+                    "Bearer test-impersonated-token"
+                ))),
+            ])
+            .times(2)
+            .respond_with(json_encoded(json!({
+                "signedBlob": BASE64_STANDARD.encode("signed_blob"),
+            }))),
+        );
+        let impersonation_url = server
+            .url("/v1/projects/-/serviceAccounts/test-principal:generateAccessToken")
+            .to_string();
+
+        // Test builder from source credentials
+        let user_credential = json!({
+            "type": "authorized_user",
+            "client_id": "test-client-id",
+            "client_secret": "test-client-secret",
+            "refresh_token": "test-refresh-token",
+            "token_uri": server.url("/token").to_string()
+        });
+        let source_credential =
+            crate::credentials::user_account::Builder::new(user_credential.clone()).build()?;
+        let mut builder_from_source = Builder::from_source_credentials(source_credential)
+            .with_target_principal("test-principal");
+        builder_from_source.service_account_impersonation_url = Some(impersonation_url.clone());
+
+        // Test builder from JSON
+        let impersonated_credential = json!({
+            "type": "impersonated_service_account",
+            "service_account_impersonation_url": impersonation_url,
+            "source_credentials": user_credential,
+        });
+        let builder_from_json = Builder::new(impersonated_credential);
+
+        for builder in [builder_from_source, builder_from_json] {
+            let iam_endpoint = server.url("").to_string().trim_end_matches('/').to_string();
+            let signer = builder.build_signer_with_iam_endpoint_override(Some(iam_endpoint))?;
+
+            let client_email = signer.client_email().await?;
+            assert_eq!(client_email, "test-principal");
+
+            let result = signer.sign(b"test").await?;
+            assert_eq!(result.as_ref(), b"signed_blob");
+        }
+
+        Ok(())
+    }
+
+    #[tokio::test]
+    async fn test_impersonated_sa_signer() -> TestResult {
+        use crate::credentials::service_account::ServiceAccountKey;
+        use crate::credentials::tests::PKCS8_PK;
+        use serde_json::Value;
+
+        let service_account = json!({
+            "type": "service_account",
+            "client_email": "test-client-email",
+            "private_key_id": "test-private-key-id",
+            "private_key": Value::from(PKCS8_PK.clone()),
+            "project_id": "test-project-id",
+        });
+        let impersonated_credential = json!({
+            "type": "impersonated_service_account",
+            "service_account_impersonation_url": "https://iamcredentials.googleapis.com/v1/projects/-/serviceAccounts/test-principal:generateAccessToken",
+            "source_credentials": service_account.clone(),
+        });
+
+        let signer = Builder::new(impersonated_credential).build_signer()?;
+
+        let client_email = signer.client_email().await?;
+        assert_eq!(client_email, "test-principal");
+
+        let result = signer.sign(b"test").await?;
+
+        let service_account_key = serde_json::from_value::<ServiceAccountKey>(service_account)?;
+        let inner_signer = service_account_key.signer().unwrap();
+        let inner_result = inner_signer.sign(b"test")?;
+        assert_eq!(result.as_ref(), inner_result);
+
+        Ok(())
+    }
+
+    #[tokio::test]
+    async fn test_impersonated_signer_with_invalid_email() -> TestResult {
+        let impersonated_credential = json!({
+            "type": "impersonated_service_account",
+            "service_account_impersonation_url": "http://example.com/test-principal:generateIdToken",
+            "source_credentials": json!({
+                "type": "service_account",
+                "client_email": "test-client-email",
+                "private_key_id": "test-private-key-id",
+                "private_key": "test-private-key",
+                "project_id": "test-project-id",
+            }),
+        });
+
+        let error = Builder::new(impersonated_credential)
+            .build_signer()
+            .unwrap_err();
+
+        assert!(error.is_parsing());
+        assert!(
+            error
+                .to_string()
+                .contains("invalid service account impersonation URL"),
+            "error: {}",
+            error
+        );
+
+        Ok(())
+    }
 }
