@@ -12,15 +12,20 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+//! The retry module
+
 use crate::errors::CredentialsError;
 use crate::token::{Token, TokenProvider};
 use crate::{Result, constants};
-use gax::backoff_policy::{BackoffPolicy, BackoffPolicyArg};
-use gax::exponential_backoff::ExponentialBackoff;
-use gax::retry_loop_internal::retry_loop;
-use gax::retry_policy::{AlwaysRetry, RetryPolicy, RetryPolicyArg, RetryPolicyExt};
-use gax::retry_throttler::{AdaptiveThrottler, RetryThrottlerArg, SharedRetryThrottler};
+use google_cloud_gax::backoff_policy::{BackoffPolicy, BackoffPolicyArg};
+use google_cloud_gax::exponential_backoff::ExponentialBackoff;
+use google_cloud_gax::retry_loop_internal::retry_loop;
+use google_cloud_gax::retry_policy::{AlwaysRetry, RetryPolicy, RetryPolicyArg, RetryPolicyExt};
+use google_cloud_gax::retry_throttler::{
+    AdaptiveThrottler, RetryThrottlerArg, SharedRetryThrottler,
+};
 use std::error::Error;
+use std::panic::{RefUnwindSafe, UnwindSafe};
 use std::sync::{Arc, Mutex};
 
 #[derive(Debug)]
@@ -38,6 +43,17 @@ pub(crate) struct Builder {
     retry_throttler: Option<RetryThrottlerArg>,
 }
 
+// This is necessary because adding [Builder] to existing, released auth features
+// caused a breaking change. The containing structs would lose their automatically derived
+// `RefUnwindSafe` implementation because the dynamic trait objects (like `dyn RetryPolicy`)
+// are not `RefUnwindSafe` nor `UnwindSafe` by default.
+//
+// This is safe because in the builder, we never call or use the retry policies. We just hold them
+// until we create the client. There is no opportunity for a panic to leave them in an inconsistent
+// state.
+impl RefUnwindSafe for Builder {}
+impl UnwindSafe for Builder {}
+
 impl Builder {
     pub(crate) fn with_retry_policy(mut self, retry_policy: RetryPolicyArg) -> Self {
         self.retry_policy = Some(retry_policy);
@@ -64,11 +80,10 @@ impl Builder {
             None => Arc::new(Mutex::new(AdaptiveThrottler::default())),
         };
 
-        let retry_policy = match self.retry_policy {
-            Some(p) => p,
-            None => AlwaysRetry.with_attempt_limit(1).into(),
-        }
-        .into();
+        let retry_policy = self
+            .retry_policy
+            .unwrap_or_else(|| AlwaysRetry.with_attempt_limit(1).into())
+            .into();
 
         TokenProviderWithRetry {
             inner: Arc::new(token_provider),
@@ -99,7 +114,7 @@ where
                 inner
                     .token()
                     .await
-                    .map_err(gax::error::Error::authentication)
+                    .map_err(google_cloud_gax::error::Error::authentication)
             }
         };
 
@@ -115,19 +130,20 @@ where
         .map_err(Self::map_retry_error)
     }
 
-    fn map_retry_error(e: gax::error::Error) -> CredentialsError {
+    fn map_retry_error(e: google_cloud_gax::error::Error) -> CredentialsError {
         if !e.is_authentication() {
             return CredentialsError::from_source(false, e);
         }
 
-        let msg = match e
+        match e
             .source()
             .and_then(|s| s.downcast_ref::<CredentialsError>())
         {
-            Some(cred_error) if cred_error.is_transient() => constants::RETRY_EXHAUSTED_ERROR,
-            _ => constants::TOKEN_FETCH_FAILED_ERROR,
-        };
-        CredentialsError::new(false, msg, e)
+            Some(cred_error) if cred_error.is_transient() => {
+                CredentialsError::new(true, constants::RETRY_EXHAUSTED_ERROR, e)
+            }
+            _ => CredentialsError::new(false, constants::TOKEN_FETCH_FAILED_ERROR, e),
+        }
     }
 }
 
@@ -136,11 +152,12 @@ mod tests {
     use super::*;
     use crate::credentials::tests::find_source_error;
     use crate::token::{Token, TokenProvider, tests::MockTokenProvider};
-    use gax::retry_policy::RetryPolicy;
-    use gax::retry_result::RetryResult;
-    use gax::retry_state::RetryState;
-    use gax::retry_throttler::RetryThrottler;
+    use google_cloud_gax::retry_policy::RetryPolicy;
+    use google_cloud_gax::retry_result::RetryResult;
+    use google_cloud_gax::retry_state::RetryState;
+    use google_cloud_gax::retry_throttler::RetryThrottler;
     use mockall::{Sequence, mock};
+    use static_assertions::assert_impl_all;
     use std::error::Error;
     use std::sync::Arc;
     use std::sync::atomic::{AtomicBool, Ordering};
@@ -163,7 +180,11 @@ mod tests {
     }
 
     impl RetryPolicy for AuthRetryPolicy {
-        fn on_error(&self, state: &RetryState, error: gax::error::Error) -> RetryResult {
+        fn on_error(
+            &self,
+            state: &RetryState,
+            error: google_cloud_gax::error::Error,
+        ) -> RetryResult {
             if state.attempt_count >= self.max_attempts {
                 return RetryResult::Exhausted(error);
             }
@@ -225,7 +246,7 @@ mod tests {
             .build(mock_provider);
 
         let token = provider.token().await.unwrap();
-        assert_eq!(token.token, "test_token");
+        assert_eq!(token.token, "test_token", "{token:?}");
     }
 
     #[tokio::test]
@@ -256,7 +277,7 @@ mod tests {
             .build(mock_provider);
 
         let token = provider.token().await.unwrap();
-        assert_eq!(token.token, "test_token");
+        assert_eq!(token.token, "test_token", "{token:?}");
     }
 
     #[tokio::test]
@@ -272,10 +293,13 @@ mod tests {
             .build(mock_provider);
 
         let error = provider.token().await.unwrap_err();
-        assert!(!error.is_transient());
+        assert!(error.is_transient(), "{error:?}");
+        assert!(
+            error.to_string().contains(constants::RETRY_EXHAUSTED_ERROR),
+            "{error:?}"
+        );
         let original_error = find_source_error::<CredentialsError>(&error).unwrap();
-        assert!(original_error.is_transient());
-        assert!(error.to_string().contains(constants::RETRY_EXHAUSTED_ERROR));
+        assert!(original_error.is_transient(), "{original_error:?}");
     }
 
     #[tokio::test]
@@ -291,14 +315,15 @@ mod tests {
             .build(mock_provider);
 
         let error = provider.token().await.unwrap_err();
-        assert!(!error.is_transient());
-        let original_error = find_source_error::<CredentialsError>(&error).unwrap();
-        assert!(!original_error.is_transient());
+        assert!(!error.is_transient(), "{error:?}");
         assert!(
             error
                 .to_string()
-                .contains(constants::TOKEN_FETCH_FAILED_ERROR)
+                .contains(constants::TOKEN_FETCH_FAILED_ERROR),
+            "{error:?}"
         );
+        let original_error = find_source_error::<CredentialsError>(&error).unwrap();
+        assert!(!original_error.is_transient(), "{original_error:?}");
     }
 
     #[tokio::test]
@@ -320,7 +345,7 @@ mod tests {
         let provider = Builder::default().build(mock_provider);
 
         let token = provider.token().await.unwrap();
-        assert_eq!(token.token, "test_token");
+        assert_eq!(token.token, "test_token", "{token:?}");
     }
 
     #[tokio::test]
@@ -336,9 +361,9 @@ mod tests {
         let provider = Builder::default().build(mock_provider);
 
         let error = provider.token().await.unwrap_err();
-        assert!(!error.is_transient());
+        assert!(error.is_transient(), "{error:?}");
         let original_error = find_source_error::<CredentialsError>(&error).unwrap();
-        assert!(original_error.is_transient());
+        assert!(original_error.is_transient(), "{original_error:?}");
     }
 
     #[tokio::test]
@@ -354,9 +379,9 @@ mod tests {
         let provider = Builder::default().build(mock_provider);
 
         let error = provider.token().await.unwrap_err();
-        assert!(!error.is_transient());
+        assert!(!error.is_transient(), "{error:?}");
         let original_error = find_source_error::<CredentialsError>(&error).unwrap();
-        assert!(!original_error.is_transient());
+        assert!(!original_error.is_transient(), "{original_error:?}");
     }
 
     #[test_case(
@@ -454,7 +479,7 @@ mod tests {
 
         // 5. Assert
         let token = provider.token().await.unwrap();
-        assert_eq!(token.token, "final_token");
+        assert_eq!(token.token, "final_token", "{token:?}");
         assert!(
             backoff_was_called.load(Ordering::SeqCst),
             "Backoff policy was not called"
@@ -464,7 +489,7 @@ mod tests {
     #[test]
     fn test_map_retry_error_non_auth_error() {
         // 1. Create a non-authentication error.
-        let original_error = gax::error::Error::io("test-io-error");
+        let original_error = google_cloud_gax::error::Error::io("test-io-error");
         let original_error_string = original_error.to_string();
 
         // 2. Call the function under test.
@@ -472,10 +497,40 @@ mod tests {
             TokenProviderWithRetry::<MockTokenProvider>::map_retry_error(original_error);
 
         // 3. Assert that the resulting error is not transient and wraps the original error.
-        assert!(!credentials_error.is_transient());
+        assert!(!credentials_error.is_transient(), "{credentials_error:?}");
         assert_eq!(
             credentials_error.source().unwrap().to_string(),
-            original_error_string
+            original_error_string,
+            "{credentials_error:?}"
         );
     }
+
+    #[test_case(false, "invalid credentials"; "permanent auth error")]
+    #[test_case(true, "transient network error"; "transient auth error")]
+    fn test_map_retry_error_auth_error(transient: bool, message: &str) {
+        // 1. Create an authentication error.
+        let error = CredentialsError::from_msg(transient, message);
+        let error = google_cloud_gax::error::Error::authentication(error);
+        let error_string = error.to_string();
+
+        // 2. Call the function under test.
+        let credentials_error = TokenProviderWithRetry::<MockTokenProvider>::map_retry_error(error);
+
+        // 3. Assert that the resulting error is transient or not like the original error and wraps the original error.
+        assert_eq!(
+            credentials_error.is_transient(),
+            transient,
+            "{credentials_error:?}"
+        );
+        assert_eq!(
+            credentials_error.source().unwrap().to_string(),
+            error_string,
+            "{credentials_error:?}"
+        );
+    }
+
+    #[test]
+    fn test_unwind_safe() {
+        assert_impl_all!(Builder: std::panic::UnwindSafe, std::panic::RefUnwindSafe);
+    }
 }
