@@ -1,5 +1,5 @@
 #[cfg(not(target_arch = "wasm32"))]
-use std::path::PathBuf;
+use std::path::{Path, PathBuf};
 
 #[cfg(not(target_arch = "wasm32"))]
 pub mod package_cache;
@@ -32,3 +32,18 @@ pub fn default_cache_dir() -> anyhow::Result<PathBuf> {
                 })
         })
 }
+
+/// Creates the cache directory if it doesn't exist and excludes it from backups.
+///
+/// This function:
+/// 1. Creates the directory and all parent directories if they don't exist
+/// 2. Creates a `CACHEDIR.TAG` file to exclude from backup tools (borg, restic, etc.)
+/// 3. On macOS, marks the directory as excluded from Time Machine
+///
+/// This is idempotent - calling it multiple times on the same directory is safe.
+#[cfg(not(target_arch = "wasm32"))]
+pub fn ensure_cache_dir(path: &Path) -> std::io::Result<()> {
+    fs_err::create_dir_all(path)?;
+    rattler_conda_types::backup::exclude_from_backups(path)?;
+    Ok(())
+}
