@@ -5,11 +5,12 @@ use nom::{
     bytes::complete::{tag, take_while},
     character::complete::{alpha1, digit1, multispace0, u32},
     combinator::{all_consuming, cut, map, not, opt, recognize, value},
-    error::{context, convert_error, ContextError, ParseError},
+    error::{context, ContextError, ParseError},
     multi::{many0, separated_list1},
-    sequence::{delimited, preceded, terminated, tuple},
-    IResult,
+    sequence::{delimited, preceded, terminated},
+    IResult, Parser,
 };
+use nom_language::error::convert_error;
 use thiserror::Error;
 
 use crate::version_spec::{
@@ -58,14 +59,15 @@ fn parse_operator<'a, E: ParseError<&'a str>>(
             VersionOperators::StrictRange(StrictRangeOperator::Compatible),
             tag("~="),
         ),
-    ))(input)
+    ))
+    .parse(input)
 }
 
 /// Recognizes the version epoch
 fn parse_version_epoch<'a, E: ParseError<&'a str> + ContextError<&'a str>>(
     input: &'a str,
 ) -> Result<(&'a str, u32), nom::Err<E>> {
-    terminated(u32, tag("!"))(input)
+    terminated(u32, tag("!")).parse(input)
 }
 
 /// A parser that recognizes a version
@@ -81,9 +83,9 @@ pub(crate) fn recognize_version<'a, E: ParseError<&'a str> + ContextError<&'a st
             let num = digit1;
             let glob = tag("*");
             if allow_glob {
-                alt((ident, num, glob))(input)
+                alt((ident, num, glob)).parse(input)
             } else {
-                alt((ident, num))(input)
+                alt((ident, num)).parse(input)
             }
         }
     }
@@ -93,18 +95,19 @@ pub(crate) fn recognize_version<'a, E: ParseError<&'a str> + ContextError<&'a st
         allow_glob: bool,
     ) -> impl FnMut(&'a str) -> IResult<&'a str, &'a str, E> {
         move |input: &'a str| {
-            recognize(tuple((
+            recognize((
                 recognize_version_component(allow_glob),
                 many0(preceded(
                     opt(take_while(|c: char| c == '.' || c == '-' || c == '_')),
                     recognize_version_component(allow_glob),
                 )),
-            )))(input)
+            ))
+            .parse(input)
         }
     }
 
     move |input: &'a str| {
-        recognize(tuple((
+        recognize((
             // Optional version epoch
             opt(context("epoch", parse_version_epoch)),
             // Version components
@@ -114,7 +117,8 @@ pub(crate) fn recognize_version<'a, E: ParseError<&'a str> + ContextError<&'a st
                 tag("+"),
                 cut(context("local", recognize_version_components(allow_glob))),
             )),
-        )))(input)
+        ))
+        .parse(input)
     }
 }
 
@@ -130,7 +134,8 @@ pub(crate) fn recognize_version_with_star<'a, E: ParseError<&'a str> + ContextEr
         ),
         // Just a *
         tag("*"),
-    ))(input)
+    ))
+    .parse(input)
 }
 
 /// A parser that recognized a constraint but does not actually parse it.
@@ -151,7 +156,8 @@ pub(crate) fn recognize_constraint<'a, E: ParseError<&'a str> + ContextError<&'a
             )),
             cut(context("version", recognize_version_with_star)),
         )),
-    ))(input)
+    ))
+    .parse(input)
 }
 
 impl<'a> TryFrom<&'a str> for VersionTree<'a> {
@@ -171,7 +177,8 @@ impl<'a> TryFrom<&'a str> for VersionTree<'a> {
                 map(recognize_constraint, |constraint| {
                     VersionTree::Term(constraint)
                 }),
-            ))(input)
+            ))
+            .parse(input)
         }
 
         /// Given multiple version tree components, flatten the structure as
@@ -199,7 +206,8 @@ impl<'a> TryFrom<&'a str> for VersionTree<'a> {
             input: &'a str,
         ) -> Result<(&'a str, VersionTree<'a>), nom::Err<E>> {
             let (rest, group) =
-                separated_list1(delimited(multispace0, tag(","), multispace0), parse_term)(input)?;
+                separated_list1(delimited(multispace0, tag(","), multispace0), parse_term)
+                    .parse(input)?;
             Ok((rest, flatten_group(LogicalOperator::And, group)))
         }
 
@@ -210,11 +218,12 @@ impl<'a> TryFrom<&'a str> for VersionTree<'a> {
             let (rest, group) = separated_list1(
                 delimited(multispace0, tag("|"), multispace0),
                 parse_and_group,
-            )(input)?;
+            )
+            .parse(input)?;
             Ok((rest, flatten_group(LogicalOperator::Or, group)))
         }
 
-        match all_consuming(parse_or_group)(input) {
+        match all_consuming(parse_or_group).parse(input) {
             Ok((_, tree)) => Ok(tree),
             Err(nom::Err::Error(e) | nom::Err::Failure(e)) => {
                 Err(ParseVersionTreeError::ParseError(convert_error(input, e)))
