@@ -45,7 +45,7 @@ pub(super) fn solve_constraints<T: SolverImpl + Default>() {
 /// Test that `exclude_newer` filters out packages newer than the given timestamp.
 pub(super) fn solve_exclude_newer<T: SolverImpl + Default>() {
     use super::helpers::run_solver_cases;
-    use rattler_conda_types::package::ArchiveType;
+    use rattler_conda_types::package::CondaArchiveType;
 
     // Basic version filtering
     let foo_old = PackageBuilder::new("foo")
@@ -63,13 +63,13 @@ pub(super) fn solve_exclude_newer<T: SolverImpl + Default>() {
     let foo_tarbz2 = PackageBuilder::new("foo")
         .version("3.0")
         .build_string("build_1")
-        .archive_type(ArchiveType::TarBz2)
+        .archive_type(CondaArchiveType::TarBz2)
         .timestamp("2021-06-01T00:00:00Z")
         .build();
     let foo_conda = PackageBuilder::new("foo")
         .version("3.0")
         .build_string("build_1")
-        .archive_type(ArchiveType::Conda)
+        .archive_type(CondaArchiveType::Conda)
         .timestamp("2022-06-01T00:00:00Z")
         .build();
     let foo_newer_version = PackageBuilder::new("foo")
