@@ -427,6 +427,13 @@ impl Default for InMemorySys {
 }
 
 impl InMemorySys {
+  pub fn new_with_cwd(cwd: impl AsRef<Path>) -> Self {
+    let sys = InMemorySys::default();
+    sys.fs_create_dir_all(cwd.as_ref()).unwrap();
+    sys.env_set_current_dir(cwd.as_ref()).unwrap();
+    sys
+  }
+
   pub fn set_seed(&self, seed: Option<u64>) {
     self.0.write().random_seed = seed;
   }
@@ -963,7 +970,7 @@ impl InMemoryDirEntry {
 impl FsDirEntry for InMemoryDirEntry {
   type Metadata = InMemoryMetadata;
 
-  fn file_name(&self) -> std::borrow::Cow<std::ffi::OsStr> {
+  fn file_name(&self) -> std::borrow::Cow<'_, std::ffi::OsStr> {
     std::borrow::Cow::Owned(self.name.clone().into())
   }
 
@@ -983,7 +990,7 @@ impl FsDirEntry for InMemoryDirEntry {
     })
   }
 
-  fn path(&self) -> std::borrow::Cow<std::path::Path> {
+  fn path(&self) -> std::borrow::Cow<'_, std::path::Path> {
     std::borrow::Cow::Borrowed(self.path.as_ref())
   }
 }
@@ -2107,6 +2114,15 @@ mod tests {
     assert!(!sys.fs_exists_no_err("/test"));
   }
 
+  #[test]
+  fn test_new_with_cwd() {
+    let cwd = if cfg!(windows) { "C:\\dir" } else { "/dir" };
+
+    let sys = InMemorySys::new_with_cwd(cwd);
+    assert_eq!(sys.env_current_dir().unwrap(), PathBuf::from(cwd));
+    assert!(sys.fs_is_dir(cwd).unwrap());
+  }
+
   #[test]
   fn test_set_permissions_all_types() {
     let sys = InMemorySys::default();
